import json
import logging
import os
from email import encoders
from email.mime.application import MIMEApplication
from email.mime.multipart import MIMEMultipart

import requests
import yaml

from datastation.common.find_bags import find_bags
from datastation.common.result_writer import ResultWriter


class ValidateDansBag:
    def __init__(self, config: dict, accept_type: str = 'application/json'):
        self.server_url = config['service_baseurl']
        self.accept_type = accept_type

    def validate(self, path: str, info_package_type: str, result_writer: ResultWriter, dry_run: bool = False):
        try:
            is_first = True
            for bag in find_bags(path, max_depth=2):
                self.validate_dans_bag(bag, info_package_type, result_writer, is_first, dry_run)
                is_first = False
        finally:
            result_writer.close()

    def validate_dans_bag(self, path: str, info_package_type: str, result_writer: ResultWriter, is_first: bool = True,
                          dry_run: bool = False):
        logging.info("Validating bag: {}".format(path))
        command = {
            'bagLocation': os.path.abspath(path),
            'packageType': info_package_type,
        }

        msg = MIMEMultipart("form-data")
        p = MIMEApplication(json.dumps(command), "json", _encoder=encoders.encode_noop)
        p.add_header("Content-Disposition", "form-data; name=command")
        msg.attach(p)

        body = msg.as_string().split('\n\n', 1)[1]
        headers = dict(msg.items())
        headers.update({'Accept': self.accept_type})

        if dry_run:
            print("Would have sent the following request:")
            print("POST {}/validate".format(self.server_url))
            return
        r = requests.post('{}/validate'.format(self.server_url), data=body,
                          headers=headers)
        if self.accept_type == 'application/json':
            result = json.loads(r.text)
        elif self.accept_type == 'text/plain':
            result = yaml.safe_load(r.text)
        else:
            raise Exception("Unknown accept type: {}".format(self.accept_type))

        result_writer.write(result, is_first)



import argparse
from datastation.managedeposit.manage_deposit import ManageDeposit
from datastation.common.config import init
from datastation.common.send_mail import SendMail


class ReportHandler:
    def __init__(self, server_url, cmd_args):
        self.__server_url = server_url
        self.__command_line_args = cmd_args

    def handle_request(self):
        report = ManageDeposit(self.__command_line_args).create_report(self.__server_url)

        if report is not None:
            output_file = self.__command_line_args.output_file
            if output_file == '-':
                print(report)
            else:
                self.save_report_to_file(report, output_file)

                email_to = self.__command_line_args.email_to
                if email_to is not None:
                    self.send_report_mail(email_to, output_file)

    def save_report_to_file(self, report, output_file):
        with open(output_file, 'w') as output:
            output.write(report)

    def send_report_mail(self, mail_to, attachment):
        SendMail.send(mail_to,
                      "Deposits report",
                      "Please, find attached the detailed report of deposits.",
                      attachment)


def main():
    config = init()
    parser = argparse.ArgumentParser(prog='deposit_create_report',
                                     description='Create and send reports based on dd-manage-deposit database')
    parser.add_argument('-o', '--output-file', dest='output_file', default='-',
                        help='the file to write the output to or - for stdout')
    parser.add_argument('-e', '--enddate', dest='enddate', help='Filter until the record creation of this date')
    parser.add_argument('-s', '--startdate', dest='startdate', help='Filter from the record creation of this date')
    parser.add_argument('-t', '--state', help='The state of the deposit')
    parser.add_argument('-u', '--user', dest='user', help='The depositor name')
    parser.add_argument('-f', '--format', dest='file_format', default='text/csv', help='Output data format')
    parser.add_argument('--email-to', dest='email_to', help='')
    args = parser.parse_args()

    server_url = config['manage_deposit']['service_baseurl'] + '/report'

    ReportHandler(server_url, args).handle_request()


if __name__ == '__main__':
    main()



import argparse
import logging
from datetime import datetime

import requests

from datastation.common.batch_processing import get_pids, BatchProcessorWithReport
from datastation.common.config import init
from datastation.common.utils import add_batch_processor_args, add_dry_run_arg
from datastation.dataverse.dataverse_client import DataverseClient


def reingest_tabular_files_in_datasets(args, dataverse_client: DataverseClient):
    pids = get_pids(args.pid_or_pid_file)
    batch_processor = BatchProcessorWithReport(report_file=args.report_file, wait=args.wait,
                                               fail_on_first_error=args.fail_fast,
                                               headers=['DOI', 'Modified', 'Change', 'Messages'])
    batch_processor.process_pids(pids,
                                 lambda pid, csv_report: reingest_tabular_files_in_dataset(pid, dataverse_client,
                                                                                           csv_report=csv_report,
                                                                                           dry_run=args.dry_run))


def reingest_tabular_files_in_dataset(pid, dataverse_client: DataverseClient, csv_report, dry_run: bool = False):
    files = dataverse_client.dataset(pid).get_files()
    try:
        for file in files:
            file_id = file['dataFile']['id']
            try:
                dataverse_client.dataset(pid).await_unlock()
                dataverse_client.file(file_id).reingest(dry_run=dry_run)
                dataverse_client.dataset(pid).await_unlock()
                csv_report.write({'DOI': pid, 'Modified': datetime.now(), 'Change': 'Re-ingested',
                                  'Messages': 'Re-ingest was requested.'})
                logging.info(f"Re-ingested file {file_id} in dataset {pid}")
            except requests.exceptions.RequestException as re:
                # if the requests throws an exception, it might be to just tell us that the file cannot be ingested
                # as tabular, or some other reason that is a valid response. In that case, we just log it and move on
                try:
                    message = re.response.json()["message"]
                    logging.info(
                        f'[{pid}] Re-ingest not completed for file id {file_id}, reason is "{message}".')
                    csv_report.write({'DOI': pid, 'Modified': datetime.now(), 'Change': 'None', 'Messages': message})
                except:
                    pass
    except Exception as e:
        csv_report.write({'DOI': pid, 'Modified': datetime.now(), 'Change': 'Error', 'Messages': str(e)})
        logging.warning(f"Error re-ingesting files in dataset {pid}: {e}. Moving on to next dataset.")
        return


def main():
    config = init()
    dataverse = DataverseClient(config['dataverse'])

    parser = argparse.ArgumentParser(description='Re-ingest the tabular data files in one or more datasets.')
    parser.add_argument('pid_or_pid_file',
                        help='the persistent identifier of the dataset or a file containing a list of '
                             'persistent identifiers.')
    add_batch_processor_args(parser)
    add_dry_run_arg(parser)
    args = parser.parse_args()
    reingest_tabular_files_in_datasets(args, dataverse)



import requests

from datastation.common.utils import print_dry_run_message


class User:

    def __init__(self, user_name, first_name, last_name, email, affiliation, position, encrypted_password):
        self.username = user_name
        self.firstName = first_name
        self.lastName = last_name
        self.email = email
        self.affiliation = affiliation
        self.position = position
        self.encrypted_password = encrypted_password

    def to_json(self):
        return {"userName": self.username, "firstName": self.firstName, "lastName": self.lastName, "email": self.email,
                "affiliation": self.affiliation, "position": self.position}

    def get_encrypted_password(self):
        return self.encrypted_password

    def __str__(self):
        return f"User(username={self.username}, firstName={self.firstName}, lastName={self.lastName}, email={self.email}, " \
               f"affiliation={self.affiliation}, position={self.position}, encrypted_password={self.encrypted_password})"


class BuiltInUsersApi:

    def __init__(self, server_url, api_token, builtin_user_key, unblock_key=None):
        self.server_url = server_url
        self.api_token = api_token
        self.builtin_users_key = builtin_user_key
        self.unblock_key = unblock_key

    def create(self, user: User, initial_password="dummy1234",
               send_email_notification=False, dry_run=False):
        url = f"{self.server_url}/api/builtin-users"
        headers = {'X-Dataverse-key': self.api_token}
        params = {'key': self.builtin_users_key,
                  'sendEmailNotification': 'true' if send_email_notification else 'false',
                  'password': initial_password}
        if self.unblock_key:
            params['unblock-key'] = self.unblock_key

        if dry_run:
            print_dry_run_message(method='POST', url=url, headers=headers, params=params, json=user.to_json())
            return None
        else:
            return requests.post(url, headers=headers, params=params, json=user.to_json())



import requests


from datastation.common.utils import print_dry_run_message


class DataverseApi:
    def __init__(self, server_url, api_token):
        self.server_url = server_url
        self.api_token = api_token

    def get_contents(self, alias="root", dry_run=False):
        headers = {"X-Dataverse-key": self.api_token}
        url = f"{self.server_url}/api/dataverses/{alias}/contents"

        if dry_run:
            print_dry_run_message(method="GET", url=url, headers=headers)
            return None

        dv_resp = requests.get(url, headers=headers)
        dv_resp.raise_for_status()

        resp_data = dv_resp.json()["data"]
        return resp_data

    def get_storage_size(self, alias="root", dry_run=False):
        """ Get dataverse storage size (bytes). """
        url = f'{self.server_url}/api/dataverses/{alias}/storagesize'
        headers = {'X-Dataverse-key': self.api_token}
        if dry_run:
            print_dry_run_message(method='GET', url=url, headers=headers)
            return None
        else:
            r = requests.get(url, headers=headers)
        r.raise_for_status()
        return r.json()['data']['message']



import argparse

import rich

from datastation.common.config import init
from datastation.common.utils import add_dry_run_arg
from datastation.dataverse.dataverse_client import DataverseClient


def add_message(args, dataverse):
    r = dataverse.banner().add(args.message, args.dismissible_by_user, dry_run=args.dry_run)
    if r is not None:
        rich.print_json(data=r.json())


def remove_message(args, dataverse):
    for msg_id in args.ids:
        r = dataverse.banner().remove(msg_id, dry_run=args.dry_run)
        if r is not None:
            rich.print_json(data=r.json())


def list_messages(args, dataverse):
    r = dataverse.banner().list(dry_run=args.dry_run)
    if r is not None:
        rich.print_json(data=r.json())


def main():
    config = init()
    dataverse = DataverseClient(config['dataverse'])

    parser = argparse.ArgumentParser()
    parser.set_defaults(func=lambda _: parser.print_help())

    subparsers = parser.add_subparsers()
    parser_add = subparsers.add_parser('add', help="Add a banner message")
    parser_add.add_argument('message', help="Message to add as banner, note that HTML can be included.")
    parser_add.add_argument('-u', '--dismissible-by-user', dest='dismissible_by_user', action='store_true',
                            help="Whether the user can permanently dismiss the banner")
    add_dry_run_arg(parser_add)
    parser_add.set_defaults(func=lambda _: add_message(_, dataverse))

    parser_remove = subparsers.add_parser('remove', help="Remove banner messages")
    parser_remove.add_argument('ids', help="One or more ids of banner messages", nargs='+')
    add_dry_run_arg(parser_remove)
    parser_remove.set_defaults(func=lambda _: remove_message(_, dataverse))

    parser_list = subparsers.add_parser('list', help="List banner messages")
    add_dry_run_arg(parser_list)
    parser_list.set_defaults(func=lambda _: list_messages(_, dataverse))

    args = parser.parse_args()
    args.func(args)


if __name__ == '__main__':
    main()



import argparse
import csv
import datetime
import json
import logging
import re

from datastation.batch_processing import batch_process
from datastation.config import init
from datastation.dv_api import publish_dataset, get_dataset_metadata, change_access_request, replace_dataset_metadata, \
    change_file_restrict


def open_access_archeodepot(datasets_file, licenses_file, must_be_restricted_files, dataverse_config, dry_run, delay):
    doi_to_license_uri = read_doi_to_license(datasets_file, read_rights_holder_to_license(licenses_file))
    doi_to_keep_restricted = read_doi_to_keep_restricted(must_be_restricted_files)
    server_url = dataverse_config['server_url']
    api_token = dataverse_config['api_token']
    logging.debug("is dry run: {}".format(dry_run))
    time_stamp = datetime.datetime.now().strftime("%Y-%m-%d_%H.%M.%S")
    batch_process(doi_to_license_uri.items(),
                  lambda doi_to_license: update_license(
                      "doi:" + doi_to_license[0],
                      doi_to_license[1],
                      doi_to_keep_restricted.get(to_key(doi_to_license[0]), []),
                      server_url,
                      api_token,
                      dry_run,
                      create_csv("datasets",
                                 ["DOI", "Modified", "OldLicense", "NewLicense", "OldRequestEnabled",
                                  "NewRequestEnabled"], time_stamp),
                      create_csv("datafiles",
                                 ["DOI", "FileID", "Modified", "OldRestricted", "NewRestricted"], time_stamp)),
                  delay)


def create_csv(datasets, fieldnames, now):
    file_name = "archeodepot-{}-{}.csv".format(datasets, now)
    csv_writer = csv.DictWriter(open(file_name, 'w'), fieldnames=fieldnames)
    csv_writer.writeheader()
    return csv_writer


def read_doi_to_license(datasets_file, rights_holder_to_license_uri):
    doi_to_license_uri = {}
    with open(datasets_file, "r") as input_file_handler:
        csv_reader = csv.DictReader(input_file_handler, delimiter=',', fieldnames=["DOI"], restkey="rest")
        for row in csv_reader:
            key = to_key(row["rest"][-1].strip())
            uri = rights_holder_to_license_uri.get(key, "")
            if uri:
                doi_to_license_uri[row["DOI"]] = uri
            else:
                logging.warning("no license for line {}: {}".format(csv_reader.line_num, row))
    return doi_to_license_uri


def read_doi_to_keep_restricted(keep_restricted_files):
    doi_to_keep_restricted = {}
    with open(keep_restricted_files, "r") as input_file_handler:
        csv_reader = csv.DictReader(input_file_handler, delimiter=',',
                                    fieldnames=["dataset_id", "DOI"], restkey="files")
        next(csv_reader)
        for row in csv_reader:
            doi_to_keep_restricted[to_key(row["DOI"])] = list(filter(lambda item: item != "", row["files"]))
    return doi_to_keep_restricted


def read_rights_holder_to_license(licenses_file):
    rights_holder_to_license_uri = {}
    with open(licenses_file, "r") as input_file_handler:
        csv_reader = csv.DictReader(input_file_handler, delimiter=',')
        for row in csv_reader:
            rights_holder_to_license_uri[to_key(row["RIGHTS_HOLDER"])] = row["URI"]
    return rights_holder_to_license_uri


def to_key(name):
    return re.sub("[^a-zA-Z0-1]", "_", name)


def update_license(doi, new_license_uri, must_be_restricted, server_url, api_token, dry_run, datasets_writer, datafiles_writer):
    resp_data = get_dataset_metadata(server_url, api_token, doi)
    change_to_restricted = list(filter(
        lambda file: not file['restricted'] and file_path(file) in must_be_restricted,
        resp_data['files']))
    change_to_accessible = list(filter(
        lambda file: file['restricted'] and file_path(file) not in must_be_restricted,
        resp_data['files']))
    logging.info("number of: must_be_restricted={}, change_to_restricted={}, change_to_accessible={}; {}".format(
        len(must_be_restricted), len(change_to_restricted), len(change_to_accessible), must_be_restricted))
    has_change_to_restricted = len(change_to_restricted) > 0
    has_must_be_restricted = len(must_be_restricted) > 0
    if has_change_to_restricted and not resp_data.get("termsOfAccess", None):
        logging.warning("no terms of access, can't change files to restricted of {}".format(doi))
        return
    dirty = False
    if bool(resp_data['fileAccessRequest']) != has_must_be_restricted:
        dirty = True
        if not dry_run:
            change_access_request(server_url, api_token, doi, has_must_be_restricted)
    old_license_uri = resp_data['license']['uri']
    if old_license_uri != new_license_uri:
        dirty = True
        if not dry_run:
            data = json.dumps({"http://schema.org/license": new_license_uri})
            replace_dataset_metadata(server_url, api_token, doi, data)
    dirty = change_file(doi, True, change_to_restricted, server_url, api_token, datafiles_writer, dry_run) or dirty
    dirty = change_file(doi, False, change_to_accessible, server_url, api_token, datafiles_writer, dry_run) or dirty
    logging.info('dirty = {} fileAccessRequest = {}, license = {}, rightsHolder = {}, title = {}'
                 .format(dirty,
                         resp_data['fileAccessRequest'],
                         resp_data['license']['name'],
                         mdb_field_value(resp_data, 'dansRights', 'dansRightsHolder'),
                         mdb_field_value(resp_data, 'citation', 'title')))
    if dirty:
        datasets_writer.writerow({"DOI": doi, "Modified": modified(),
                                  "OldLicense": old_license_uri,
                                  "NewLicense": new_license_uri,
                                  "OldRequestEnabled": resp_data['fileAccessRequest'],
                                  "NewRequestEnabled": has_must_be_restricted})
    if dirty and not dry_run:
        logging.info(doi + ' publish_dataset')
        publish_dataset(server_url, api_token, doi, 'updatecurrent')


def file_path(file_item):
    return re.sub("^/", "", file_item.get('directoryLabel', "") + "/" + file_item['label'])


def change_file(doi, restricted_value: bool, files, server_url, api_token, datafiles_writer, dry_run):
    if len(files) == 0:
        return False
    else:
        for file_id in list(map(lambda file: file['dataFile']['id'], files)):
            logging.debug("updating {}".format(file_id))
            datafiles_writer.writerow(
                {"DOI": doi, "FileID": file_id, "Modified": modified(),
                 "OldRestricted": not restricted_value,
                 "NewRestricted": restricted_value})
            if not dry_run:
                change_file_restrict(server_url, api_token, file_id, restricted_value)
        return True


def modified():
    return datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S.%f")


def mdb_field_value(resp_data, metadata_block, field_name):
    return next(filter(lambda m: m['typeName'] == field_name,
                       resp_data['metadataBlocks'][metadata_block]['fields']
                       ))['value']


def main():
    config = init()
    parser = argparse.ArgumentParser(description='Change archeodepot dataset to open access')
    parser.add_argument('-d', '--datasets', dest='datasets',
                        help='CSV file (solr query result) header: DOI, ..., RIGHTS_HOLDER')
    parser.add_argument('-r', '--dag-rapporten', dest='dag_rapporten',
                        help='CSV file with header: dataset_id, DOI, File1, File2... N.B. The DOI is just the id, not a uri')
    parser.add_argument('-l', '--licenses', dest='licenses',
                        help='CSV file with: uri, name. N.B. no trailing slash for the uri')
    parser.add_argument('--delay', default=5.0,
                        help="Delay in seconds (publish does a lot after the asynchronous request is returning)")
    parser.add_argument('--dry-run', dest='dry_run', action='store_true',
                        help="only logs the actions, nothing is executed")

    args = parser.parse_args()
    open_access_archeodepot(args.datasets, args.licenses, args.dag_rapporten, config['dataverse'], args.dry_run,
                            float(args.delay))


if __name__ == '__main__':
    main()



import csv
import logging
import sys


class CsvReport:
    """A simple, self-closing wrapper around csv.DictWriter to make it easier to use."""

    def __init__(self, filename, headers):
        self.filename = filename
        self.headers = headers
        self.csv_file = sys.stdout if filename == '-' else open(filename, 'w')
        self.csv_writer = csv.DictWriter(self.csv_file, headers, lineterminator='\n')
        self.csv_writer.writeheader()

    def write(self, row):
        logging.debug(f"Writing row: {row}")
        self.csv_writer.writerow(row)

    def close(self):
        if self.filename != '-':
            self.csv_file.close()

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.close()


class CsvInput:
    """A simple, self-closing wrapper around csv.DictReader to make it easier to use."""

    def __init__(self, filename, delimiter=','):
        self.filename = filename
        self.csv_file = sys.stdin if filename == '-' else open(filename, 'r')
        self.csv_reader = csv.DictReader(self.csv_file, delimiter=delimiter)

    def __iter__(self):
        return self.csv_reader.__iter__()

    def close(self):
        if self.filename != '-':
            self.csv_file.close()

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.close()



import argparse

from datastation.common.config import init
from datastation.common.utils import add_dry_run_arg
from datastation.dataverse.dataverse_client import DataverseClient
from datastation.dataverse.user_import import UserImport


def main():
    config = init()

    parser = argparse.ArgumentParser(description='Import users from a CSV file')
    parser.add_argument('--easy', dest='is_easy_format',
                        help="The csv file is exported from EASY and has the following columns: UID, INITIALS, SURNAME,"
                             "PREFIX, EMAIL, ORGANISATION, FUNCTION, PASSWORD-HASH. "
                             "If not set, the following columns are expected: Username, GivenName, FamilyName, Email, "
                             "Affiliation, Position, encryptedpassword",
                        action='store_true')
    parser.add_argument('-k', '--builtin-users-key', help="BuiltinUsers.KEY set in Dataverse")
    parser.add_argument('-i', '--input-csv', help="the csv file containing the users and hashed passwords")
    add_dry_run_arg(parser)

    args = parser.parse_args()
    dataverse_client = DataverseClient(config['dataverse'])
    user_import = UserImport(dataverse_client, builtin_users_key=args.builtin_users_key,
                             is_easy_format=args.is_easy_format, dry_run=args.dry_run)
    user_import.import_users(args.input_csv)


if __name__ == '__main__':
    main()



from datastation.common.result_writer import CsvResultWriter, YamlResultWriter, JsonResultWriter
from datastation.dataverse.dataverse_client import DataverseClient
import logging
import re
import csv
import sys
import json
import rich
from datetime import timedelta


def extract_size_str(msg):
    # Example message: "Total size of the files stored in this dataverse: 43,638,426,561 bytes"
    # try parsing the size from this string.
    size_found = re.search('dataverse: (.+?) bytes', msg).group(1)
    # remove those ',' delimiters and optionally '.' as well,
    # depending on locale (there is no fractional byte!).
    # Delimiters are probably there to improve readability of those large numbers,
    # but calculating with it is problematic.
    clean_size_str = size_found.translate({ord(i): None for i in ',.'})
    return clean_size_str


class MetricsCollect:

    def __init__(self, dataverse_client: DataverseClient, output_file, output_format, dry_run: bool = False):
        self.dataverse_client = dataverse_client
        self.output_file = output_file
        self.output_format = output_format
        self.dry_run = dry_run

        self.writer = None
        self.is_first = True  # Would be nicer if the Writer does the bookkeeping

    def create_result_writer(self, out_stream):
        logging.info(f'Writing output: {self.output_file}, with format : {self.output_format}')
        csv_columns = ['depth', 'parentalias', 'alias', 'name', 'storagesize']
        if self.output_format == 'csv':
            return CsvResultWriter(headers=csv_columns, out_stream=out_stream)
        else:
            return JsonResultWriter(out_stream)

    def write_result_row(self, row):
        self.writer.write(row, self.is_first)
        self.is_first = False  # Only the first time it can be True

    def get_result_row(self, parent_alias, child_alias, child_name, depth):
        logging.info(f'Retrieving size for dataverse: {parent_alias} / {child_alias} ...')
        msg = self.dataverse_client.dataverse().get_storage_size(child_alias)
        storage_size = extract_size_str(msg)
        logging.info(f'size: {storage_size}')
        row = {'depth': depth, 'parentalias': parent_alias, 'alias': child_alias, 'name': child_name,
               'storagesize': storage_size}
        return row

    # Traverses the tree and collects sizes for each dataverse using recursion.
    # Note that storing the parents size if all children sizes are also stored is redundant.
    def collect_children_sizes(self, parent_data, max_depth, depth=1):
        parent_alias = parent_data['alias']
        # Only direct descendants (children)
        if 'children' in parent_data:
            for child_data in parent_data['children']:
                row = self.get_result_row(parent_alias, child_data['alias'], child_data['name'], depth)
                self.write_result_row(row)

                if depth < max_depth:
                    self.collect_children_sizes(child_data, max_depth, depth + 1)  # Recurse

    def collect_storage_usage(self, max_depth=1, include_grand_total: bool = False):
        out_stream = sys.stdout
        if self.output_file != '-':
            try:
                out_stream = open(self.output_file, "w")
            except:
                logging.error(f"Could not open file: {self.output_file}")
                raise

        self.writer = self.create_result_writer(out_stream)

        logging.info(f'Extracting tree for server: {self.dataverse_client.server_url} ...')
        tree_data = self.dataverse_client.metrics().get_tree()
        alias = tree_data['alias']
        name = tree_data['name']
        logging.info(f'Extracted the tree for the toplevel dataverse: {name} ({alias})')

        if include_grand_total:
            logging.info("Retrieving the total size for this dataverse instance...")
            row = self.get_result_row("-", alias, name, 0)  # The root has no parent
            self.write_result_row(row)

        self.collect_children_sizes(tree_data, max_depth, 1)
        self.writer.close()
        self.is_first = True



import logging

import requests

from datastation.common.utils import print_dry_run_message


class SearchApi:

    def __init__(self, server_url, api_token):
        self.url = f"{server_url}/api/search"
        self.api_token = api_token

    def search(self, query="*", subtree="root", object_type="dataset", dry_run=False, rows=0, start=0):
        """
        Do a query via the public search API, only published datasets
        using the public search 'API', so no token needed

        Note that the current functionality of this function is very limited!

        :param query:
        :param subtree: This is the collection (dataverse alias)
                        it recurses into a collection and its children etc. very useful with nesting collection
        :param object_type:
        :param dry_run: Do not perform the action, but show what would be done.
        :param start: The cursor (zero based result index) indicating where the result page starts
        :param rows: The number of results returned in the 'page'
                     if zero: 25 rows at a time are read until no more rows are found
        :return: The search results in a list of dictionaries.
                 Make sure the result is not transformed to an array before feeding it to a batch processor,
                 otherwise all pages are read before processing starts. In other words:
                 don't: [record['global_id'] for record in dataverse_client.search_api().search()]
                 but: map(lambda rec: rec['global_id'], dataverse_client.search_api().search())
        """

        if rows == 0:
            per_page = 25
        else:
            per_page = rows

        params = {
            "q": query,
            "subtree": subtree,
            "type": object_type,
            "per_page": str(per_page),
            "start": str(start),
        }

        headers = {"X-Dataverse-key": self.api_token}

        if dry_run:
            print_dry_run_message(method="GET", url=self.url, headers=headers, params=params)
            return None

        while True:
            dv_resp = requests.get(self.url, headers=headers, params=params)
            dv_resp.raise_for_status()

            data = dv_resp.json()["data"]
            items = data["items"]
            logging.debug(f"{len(items)} items, {params}")

            for item in items:
                logging.debug(f"ITEM: {item}")
                yield item

            if len(items) < per_page or rows != 0:
                break

            start += per_page
            params["start"] = str(start)



import logging

from datastation.dataverse.dataverse_client import DataverseClient


class Datasets:

    def __init__(self, dataverse_client: DataverseClient, dry_run: bool = False):
        self.dataverse_client = dataverse_client
        self.dry_run = dry_run

    def get_dataset_attributes(self, pid: str,  storage: bool = False, user_with_role: str = None):
        logging.debug(f"pid={pid}")
        attributes = {"pid": pid}

        dataset_api = self.dataverse_client.dataset(pid)
        if storage:
            dataset = dataset_api.get(dry_run=self.dry_run)
            attributes["storage"] = sum(
                f["dataFile"]["filesize"] for f in dataset["files"]
            )

        if user_with_role is not None:
            role_assignments = dataset_api.get_role_assignments(dry_run=self.dry_run)
            attributes["users"] = [
                user["assignee"].replace("@", "")
                for user in role_assignments
                if user["_roleAlias"] == user_with_role
            ]

        return attributes



from datastation.common.database import Database
from datastation.dataverse.banner_api import BannerApi
from datastation.dataverse.builtin_users import BuiltInUsersApi
from datastation.dataverse.dataset_api import DatasetApi
from datastation.dataverse.dataverse_api import DataverseApi
from datastation.dataverse.file_api import FileApi
from datastation.dataverse.metrics_api import MetricsApi
from datastation.dataverse.search_api import SearchApi


class DataverseClient:
    """ A client for the Dataverse API. """

    def __init__(self, config: dict):
        self.server_url = config['server_url']
        self.api_token = config['api_token']
        self.unblock_key = config['unblock_key'] if 'unblock_key' in config else None
        self.safety_latch = config['safety_latch']
        self.db_config = config['db']

    def banner(self):
        return BannerApi(self.server_url, self.api_token, self.unblock_key)

    def search_api(self):
        return SearchApi(self.server_url, self.api_token)

    def dataset(self, pid):
        return DatasetApi(pid, self.server_url, self.api_token, self.unblock_key, self.safety_latch)

    def dataverse(self):
        return DataverseApi(self.server_url, self.api_token)

    def file(self, file_id):
        return FileApi(file_id, self.server_url, self.api_token, self.unblock_key, self.safety_latch)

    def built_in_users(self, builtin_users_key):
        return BuiltInUsersApi(self.server_url, self.api_token, builtin_users_key, self.unblock_key)

    def database(self):
        return Database(self.db_config)

    def metrics(self):
        return MetricsApi(self.server_url)



import argparse
from datastation.managedeposit.manage_deposit import ManageDeposit
from datastation.common.config import init


def clean_manage_deposit_data(server_url, args):
    result = ManageDeposit(args).clean_data(server_url)
    if result is not None:
        print(result)


def main():
    config = init()
    parser = argparse.ArgumentParser(prog='deposit_data_cleaner', description='Clean up dd-manage-deposit database')
    parser.add_argument('-e', '--enddate', dest='enddate', help='Filter until the record creation of this date')
    parser.add_argument('-s', '--startdate', dest='startdate', help='Filter from the record creation of this date')
    parser.add_argument('-t', '--state', dest='state', help='The state of the deposit')
    parser.add_argument('-u', '--user', dest='user', help='The depositor name')
    args = parser.parse_args()

    server_url = config['manage_deposit']['service_baseurl'] + '/delete-deposit'

    clean_manage_deposit_data(server_url, args)


if __name__ == '__main__':
    main()



import argparse
import sys

from datastation.common.result_writer import CsvResultWriter, JsonResultWriter, YamlResultWriter
from datastation.common.utils import add_dry_run_arg
from datastation.common.config import init
from datastation.dans_bag.validate_dans_bag import ValidateDansBag


def create_result_writer(file_format):
    if file_format == 'csv':
        return CsvResultWriter(headers=['Bag location', 'Information package type', 'Is compliant',
                                        'Name', 'Profile version', 'Rule violations'],
                               out_stream=sys.stdout)
    elif file_format == 'yaml':
        return YamlResultWriter(out_stream=sys.stdout)
    else:
        return JsonResultWriter(out_stream=sys.stdout)


def main():
    config = init()

    default_information_package_type = config['validate_dans_bag']['default_information_package_type']
    parser = argparse.ArgumentParser(
        description='Validate one or more bags to see if they comply with the DANS BagIt Profile v1')
    parser.add_argument('path', metavar='<batch-or-deposit-or-bag>',
                        help='Directory containing a bag, a deposit or a batch of deposits')
    parser.add_argument('-t', '--information-package-type', dest='info_package_type',
                        help=f'Which information package type to validate this bag as (default: '
                             f'{default_information_package_type})',
                        choices=['DEPOSIT', 'MIGRATION'],
                        default=default_information_package_type)
    parser.add_argument('-f', '--format', dest='format',
                        help='Output format, one of: csv, json, yaml (default: json)')
    parser.add_argument('-a', '--accept', dest='accept', default='application/json',
                        help='Accept header to send to server. Note that the server only supports application/json and'
                             'text/plain, the latter will return YAML. This option is only useful for debugging '
                             'purposes. (default: application/json)')
    add_dry_run_arg(parser)

    args = parser.parse_args()
    validate_dans_bag = ValidateDansBag(config['validate_dans_bag'], args.accept)
    validate_dans_bag.validate(args.path, args.info_package_type, create_result_writer(args.format),
                               dry_run=args.dry_run)


if __name__ == '__main__':
    main()



import argparse

from datastation.common.config import init
from datastation.ingestflow.ingest_flow import IngestFlow


def main():
    config = init()
    ingest_flow = IngestFlow(config['ingest_flow'])

    parser = argparse.ArgumentParser(description='Commands to control the ingest flow')
    parser.set_defaults(func=lambda _: parser.print_help())
    parser.add_argument('-d', '--dry-run', dest='dry_run', action='store_true',
                        help='Only print command to be sent to server, but do not actually send it')

    subparsers = parser.add_subparsers(help='sub-command help', dest='command')

    parser_start_migration = subparsers.add_parser('start-migration', help='Start migration of deposit or batch of '
                                                                           'deposits')
    parser_start_migration.add_argument('deposit_path', metavar='<batch-or-deposit>',
                                        help='Directory containing one deposit or a batch of deposits')
    parser_start_migration.add_argument('-s', '--single', dest="single_deposit", action="store_true",
                                        help="<batch-or-deposit> refers to a single deposit")
    parser_start_migration.add_argument('-c', '--continue', dest='continue_previous', action='store_true',
                                        help="continue previously stopped batch "
                                             "(i.e. allow output directory to be non-empty)")
    parser_start_migration.set_defaults(func=lambda _: ingest_flow.start_ingest(args.deposit_path, args.single_deposit,
                                                                                args.continue_previous,
                                                                                is_migration=True))

    parser_start_import = subparsers.add_parser('start-import', help='Start import of deposit or batch of deposits')
    parser_start_import.add_argument('deposit_path', metavar='<batch-or-deposit>',
                                     help='Directory containing one deposit or a batch of deposits')
    parser_start_import.add_argument('-s', '--single', dest="single_deposit", action="store_true",
                                     help="<batch-or-deposit> refers to a single deposit")
    parser_start_import.add_argument('-c', '--continue', dest='continue_previous', action='store_true',
                                     help="continue previously stopped batch "
                                          "(i.e. allow output directory to be non-empty)")
    parser_start_import.set_defaults(func=lambda _: ingest_flow.start_ingest(args.deposit_path, args.single_deposit,
                                                                             args.continue_previous,
                                                                             is_migration=False))

    parser_block_target = subparsers.add_parser('block-target', help='Block target')
    parser_block_target.add_argument('target', metavar='<target>', help='Target to block')
    parser_block_target.set_defaults(func=lambda _: ingest_flow.block_target(args.target))

    parser_unblock_target = subparsers.add_parser('unblock-target', help='Unblock target')
    parser_unblock_target.add_argument('target', metavar='<target>', help='Target to unblock')
    parser_unblock_target.set_defaults(func=lambda _: ingest_flow.unblock_target(args.target))

    parser_list_events = subparsers.add_parser('list-events', help='List events')
    group = parser_list_events.add_mutually_exclusive_group()
    group.add_argument('-s', '--source', dest='source', help='Source to filter on')
    group.add_argument('-d', '--deposit', dest='deposit', help='UUID of deposit to filter on')
    parser_list_events.set_defaults(func=lambda _: ingest_flow.list_events(args.source, args.deposit))

    parser_progress_report = subparsers.add_parser('progress-report', help='Progress report')
    parser_progress_report.add_argument('batch', metavar='<batch>', help='Batch to report on')
    parser_progress_report.set_defaults(func=lambda _: ingest_flow.progress_report(args.batch))

    parser_copy_batch = subparsers.add_parser('copy-batch', help='Copy batch to ingest area.')
    parser_copy_batch.add_argument('batch', metavar='<batch>', help='Batch to copy')
    parser_copy_batch.add_argument('target', metavar='<target>', help='Target to copy to')
    parser_copy_batch.set_defaults(func=lambda _: ingest_flow.copy_batch_to_ingest_area(args.batch, args.target))

    args = parser.parse_args()
    ingest_flow.set_dry_run(args.dry_run)

    args.func(args)


if __name__ == '__main__':
    main()



import requests

from datastation.common.utils import print_dry_run_message


class VerifyDatasetService:

    def __init__(self, config):
        self.base_url = config['url']

    def verify_dataset(self, pid, dry_run=False):
        url = f'{self.base_url}/verify'
        json = {'datasetPid': pid}
        headers = {'Content-Type': 'application/json'}
        if dry_run:
            print_dry_run_message(method='POST', url=url, headers=headers, json=json)
            return None
        else:
            r = requests.post(url, headers=headers, json=json)

        return r.json()



import json

import requests


class MetricsApi:
    def __init__(self, server_url: str):
        self.server_url = server_url

    def get_tree(self, dry_run: bool = False):
        """ Get dataverses hierarchy (tree). """
        url = f'{self.server_url}/api/info/metrics/tree'
        if dry_run:
            print(f"Would have sent the following request: {url}")
            return
        r = requests.get(url)
        r.raise_for_status()
        return r.json()['data']



import argparse
import json

from datastation.common.batch_processing import get_pids, BatchProcessor, BatchProcessorWithReport
from datastation.common.config import init
from datastation.common.utils import add_batch_processor_args, add_dry_run_arg
from datastation.dataverse.datasets import Datasets
from datastation.dataverse.dataverse_client import DataverseClient


def main():
    config = init()
    parser = argparse.ArgumentParser(description="Retrieves attributes of a dataset")

    attr_group = parser.add_argument_group()
    attr_group.add_argument("--user-with-role", dest="user_with_role",
                        help="List users with a specific role on the dataset",)
    attr_group.add_argument("--storage", dest="storage", action="store_true",
                        help="The storage in bytes",)

    group = parser.add_mutually_exclusive_group(required=True)

    group.add_argument('pid_or_pids_file', nargs="?",
                       help="The dataset pid, or a file with a list of pids", )
    group.add_argument("--all", dest="all_datasets", action="store_true", required=False,
                       help="All datasets in the dataverse", )

    add_batch_processor_args(parser, report=False)
    add_dry_run_arg(parser)

    args = parser.parse_args()

    attribute_options = {
        'storage': args.storage,
        'user_with_role': args.user_with_role,
    }
    if set(attribute_options.values()) == {None, False}:
        parser.error(f"Add at least one of the arguments: {', '.join(attribute_options.keys())}")

    dataverse_client = DataverseClient(config["dataverse"])

    datasets = Datasets(dataverse_client, dry_run=args.dry_run)
    BatchProcessor(wait=args.wait, fail_on_first_error=args.fail_fast).process_pids(
        get_pids(args.pid_or_pids_file, dataverse_client.search_api(), dry_run=args.dry_run),
        lambda pid: print(json.dumps(datasets.get_dataset_attributes(pid, **attribute_options), skipkeys=True)))


if __name__ == "__main__":
    main()



from datastation.common.csv import CsvInput
from datastation.common.database import Database
from datastation.dataverse.builtin_users import User
from datastation.dataverse.dataverse_client import DataverseClient
import logging
from datetime import timedelta


class Notifications:

    def __init__(self, dataverse_client: DataverseClient, dry_run: bool = False):
        self.dataverse_client = dataverse_client
        self.dry_run = dry_run

    def cleanup(self, days_old):
        logging.info("Cleaning up notifications older than %s days", days_old)

        with self.dataverse_client.database() as database:
            notifications_older_than = timedelta(days=days_old)

            delete_statement = """
                delete
                from usernotification
                where senddate < now() - %s
            """

            if self.dry_run:
                logging.info(f"dry-run, not updating database with {delete_statement % (notifications_older_than,)}")
                return

            amount_deleted = database.update(delete_statement, (notifications_older_than,))
            logging.info("Deleted %s notifications", amount_deleted)






import argparse
import json
import sys

import rich

from datastation.common.config import init
from datastation.common.utils import add_dry_run_arg
from datastation.dataverse.dataverse_client import DataverseClient


def main():
    config = init()
    parser = argparse.ArgumentParser(description='Retrieves the native format for a dataset version')
    parser.add_argument('pid', help='the pid of the dataset')
    parser.add_argument('-v', '---version', dest="version", default=':latest', help='the version of the dataset')
    parser.add_argument('-o', '--output-file', dest='output_file', default='-',
                        help='the file to write the output to or - for stdout')
    add_dry_run_arg(parser)

    args = parser.parse_args()
    dataverse = DataverseClient(config['dataverse'])
    try:
        metadata = dataverse.dataset(args.pid).get_metadata(args.version, dry_run=args.dry_run)
        if args.output_file == '-':
            rich.print_json(data=metadata)
        else:
            with open(args.output_file, 'w') as f:
                json.dump(metadata, f)
        print(f"Retrieved metadata for dataset {args.pid} version {args.version}", file=sys.stderr)
    except Exception as e:
        print(f"Error retrieving metadata: {e}")


if __name__ == '__main__':
    main()



import argparse

import rich

from datastation.common.config import init
from datastation.common.utils import add_dry_run_arg
from datastation.dataverse.dataverse_client import DataverseClient


def add_lock(args, dataverse_client: DataverseClient):
    r = dataverse_client.dataset(args.pid).add_lock(args.lock_type, dry_run=args.dry_run)
    if r is not None:
        rich.print_json(data=r)


def remove_lock(args, dataverse_client: DataverseClient):
    if args.lock_type == 'all':
        r = dataverse_client.dataset(args.pid).remove_all_locks(dry_run=args.dry_run)
    else:
        r = dataverse_client.dataset(args.pid).remove_lock(args.lock_type, dry_run=args.dry_run)
    if r is not None:
        rich.print_json(data=r)


def list_locks(args, dataverse_client: DataverseClient):
    r = dataverse_client.dataset(args.pid).get_locks()
    if r is not None:
        rich.print_json(data=r)


def main():
    config = init()
    dataverse = DataverseClient(config['dataverse'])
    lock_types = ['Ingest', 'Workflow', 'InReview', 'DcmUpload', 'finalizePublication', 'EditInProgress',
                  'FileValidationFailed']

    parser = argparse.ArgumentParser(description='Manage locks on datasets. Locks can have the following types: '
                                                 ', '.join(lock_types) + '.')

    subparsers = parser.add_subparsers(help='subcommands', dest='subcommand')

    parser_add = subparsers.add_parser('add', help='add a lock to a dataset.')
    parser_add.add_argument('lock_type', help='the type of lock to add.', choices=lock_types)
    parser_add.add_argument('pid', help='the persistent identifier of the dataset.')
    add_dry_run_arg(parser_add)
    parser_add.set_defaults(func=add_lock)

    parser_remove = subparsers.add_parser('remove', help='remove a lock from a dataset.')
    parser_remove.add_argument('lock_type', help='the type of lock to remove or all to remove all locks.',
                               choices=lock_types + ['all'])
    parser_remove.add_argument('pid', help='the persistent identifier of the dataset.')
    add_dry_run_arg(parser_remove)
    parser_remove.set_defaults(func=remove_lock)

    parser_list = subparsers.add_parser('list', help='list locks on a dataset.')
    parser_list.add_argument('pid', help='the persistent identifier of the dataset.')
    add_dry_run_arg(parser_list)
    parser_list.set_defaults(func=list_locks)

    args = parser.parse_args()
    args.func(args, dataverse)


if __name__ == '__main__':
    main()



import os
from re import match


def rpm_qa():
    return os.popen('rpm -qa')


evr_pattern = r'(?P<name>.*?)-(?P<version>\d+\.\d+\.\d+)-(?P<release>\d+)'
payara_version_pattern = r'Thank you for downloading Payara Server (.*).'


def get_rpm_versions(prefix):
    """Get the versions of the RPMs installed on the system."""
    rpm_versions = {}
    for line in rpm_qa():
        if line.startswith(prefix):
            evr = match(evr_pattern, line)
            version = evr.group('version')
            module = evr.group('name')
            rpm_versions[module] = version

    return rpm_versions


def get_dataverse_version(dataverse_application_path):
    with open(os.path.join(dataverse_application_path, 'WEB-INF', 'classes', 'META-INF',
                           'microprofile-config.properties'), 'r') as f:
        for line in f:
            if 'dataverse.version' in line:
                return (line.split('=')[1]).strip()


def get_dataverse_build_number(dataverse_application_path):
    with open(os.path.join(dataverse_application_path, 'WEB-INF', 'classes', 'BuildNumber.properties'), 'r') as f:
        for line in f:
            if 'build.number' in line:
                return (line.split('=')[1]).strip()


def get_payara_version(payara_application_path):
    with open(os.path.join(payara_application_path, 'README.txt'), 'r') as f:
        # Find first line that matches pattern
        line = next((line for line in f if match(payara_version_pattern, line)), None)
        # get subgroup 1 of the match
        payara_version = match(payara_version_pattern, line).group(1)
    return payara_version



import logging
import os
from logging import config as logconfig
from os.path import exists
from pkgutil import get_data

import yaml

configuration_file = '.dans-datastation-tools.yml'
example_configuration_file = 'example-dans-datastation-tools.yml'
configuration_file_locations = [configuration_file, os.path.expanduser('~/' + configuration_file)]


def ensure_configuration_file_exists():
    config = find_config_file()
    if config is None:
        print("No %s found; instantiating in current directory" % configuration_file)
        with open(configuration_file, 'wb') as f:
            example_cfg = get_data('datastation', example_configuration_file)
            if example_cfg is None:
                print("WARN: cannot find example-dans-datastation-tools.yml")
            else:
                f.write(example_cfg)
                f.flush()
                logging.debug("Make sure only user can read and write configuration file")
                os.chmod(path=configuration_file, mode=0o700)


def find_config_file():
    return next(filter(lambda p: exists(p), configuration_file_locations), None)


def init():
    """
    Initialization function to run by each script. It reads `.dans-datastation-tools.yml`, searching the current
    working directory and then the user's home directory. If `.dans-datastation-tools.yml` does not exist yet,
    then it is first instantiated in the current working directory, from `example-dans-datastation-tools.yml`.

    This function then proceeds to read the configuration into a dictionary, initialize the logging framework with the
    settings found under the `logging` key and return the complete dictionary to the caller.

    Returns:
        a dictionary with the configuration settings
    """
    ensure_configuration_file_exists()
    with open(find_config_file(), 'r') as stream:
        config = yaml.safe_load(stream)
        logconfig.dictConfig(config['logging'])
        logging.debug("Initialized logging")
        return config



import requests

from datastation.common.utils import print_dry_run_message


class FileApi:

    def __init__(self, id, server_url, api_token, unblock_key, safety_latch):
        self.id = id
        self.server_url = server_url
        self.api_token = api_token
        self.unblock_key = unblock_key
        self.safety_latch = safety_latch

    def reingest(self, dry_run=False):
        url = f'{self.server_url}/api/files/{self.id}/reingest'
        params = {'unblock-key': self.unblock_key}
        headers = {'X-Dataverse-key': self.api_token}
        if dry_run:
            print_dry_run_message(method='POST', url=url, headers=headers, params=params)
            return None
        r = requests.post(url, headers=headers, params=params)
        r.raise_for_status()
        return r



import argparse
import json

from requests import HTTPError

from datastation.common.batch_processing import get_pids, BatchProcessorWithReport
from datastation.common.config import init
from datastation.common.csv import CsvReport
from datastation.common.utils import add_batch_processor_args, add_dry_run_arg
from datastation.dataverse.dataverse_client import DataverseClient


def update_datacite_records(args, dataverse_client: DataverseClient):
    pids = get_pids(args.pid_or_pids_file)
    batch_processor = BatchProcessorWithReport(wait=args.wait, fail_on_first_error=args.fail_fast,
                                               report_file=args.report_file, headers=["PID", "Status", "Message"])
    batch_processor.process_pids(pids,
                                 lambda pid, csv_report: update_datacite_record(pid, dataverse_client,
                                                                                csv_report=csv_report,
                                                                                dry_run=args.dry_run))


def update_datacite_record(pid, dataverse_client: DataverseClient, csv_report: CsvReport, dry_run=False):
    try:
        r = dataverse_client.dataset(pid).modify_registration_metadata(dry_run=dry_run)
        csv_report.write({"PID": pid, "Status": "200", "Message": json.dumps(r)})
    except HTTPError as e:
        csv_report.write({"PID": pid, "Status": e.response.status_code, "Message": e.response.text})


def main():
    config = init()
    dataverse = DataverseClient(config['dataverse'])

    parser = argparse.ArgumentParser(description='Updates the DataCite records for the PIDs in the input')
    parser.add_argument('pid_or_pids_file', help='PID or newline separated file with PIDs')
    add_batch_processor_args(parser)
    add_dry_run_arg(parser)

    args = parser.parse_args()
    update_datacite_records(args, dataverse_client=dataverse)


if __name__ == '__main__':
    main()



import argparse


from datastation.common.config import init
from datastation.common.utils import add_dry_run_arg
from datastation.dataverse.dataverse_client import DataverseClient
from datastation.dataverse.metrics_collect import MetricsCollect


def main():
    config = init()

    parser = argparse.ArgumentParser(description='Collect the storage usage for the dataverses')
    parser.add_argument('-m', '--max-depth', dest='max_depth', type=int, default=1,
                        help='the max depth of the hierarchy to traverse')
    parser.add_argument('-g', '--include-grand-total', dest='include_grand_total', action='store_true',
                        help='whether to include the grand total, which almost doubles server processing time')
    parser.add_argument('-o', '--output-file', dest='output_file', default='-',
                        help='the file to write the output to or - for stdout')
    parser.add_argument('-f', '--format', dest='format',
                        help='Output format, one of: csv, json (default: json)')

    add_dry_run_arg(parser)
    args = parser.parse_args()

    dataverse_client = DataverseClient(config['dataverse'])
    collector = MetricsCollect(dataverse_client, args.output_file, args.format, args.dry_run)
    collector.collect_storage_usage(args.max_depth, args.include_grand_total)

if __name__ == '__main__':
    main()



import requests


class ManageDeposit:
    """ Get python script input arguments and
            convert them to HTTP request parameters and headers and
            submit the request"""

    def __init__(self, cmd_args):
        self.__cmd_args = cmd_args
        self.__payload = {'user': cmd_args.user, 'state': cmd_args.state,
                          'startdate': cmd_args.startdate, 'enddate': cmd_args.enddate}
        self.__headers = dict()


    def compose_headers(self):
        """ Build an HTTP __headers dictionary to send with the class:`Requests` """
        if self.__cmd_args.file_format is not None:
            self.__headers['Accept'] = str(self.__cmd_args.file_format)

    def create_report(self, server_url):
        self.compose_headers()
        response = requests.get(server_url, self.__payload, headers=self.__headers)

        if response.status_code == requests.codes.ok:
            return response.text

        print("Error: ManageDeposit:create_report() - response.status_code" + str(response.status_code))
        return None

    def clean_data(self, server_url):
        response = requests.post(server_url, params=self.__payload)

        if response.status_code == requests.codes.ok:
            return response.text

        print("Error: ManageDeposit:clean_datat() - response.status_code" + str(response.status_code))
        return None



import argparse
from datetime import datetime

from datastation.common.batch_processing import BatchProcessor, get_pids, BatchProcessorWithReport
from datastation.common.config import init
from datastation.common.utils import add_batch_processor_args, add_dry_run_arg
from datastation.dataverse.dataverse_client import DataverseClient


def destroy_datasets(args, dataverse_client: DataverseClient, batch_processor: BatchProcessor, dry_run: bool):
    pids = get_pids(args.pid_or_pid_file)
    batch_processor.process_pids(pids, lambda pid, csv_report: destroy_dataset(pid,
                                                                               dataset_api=dataverse_client.dataset(
                                                                                   pid),
                                                                               csv_report=csv_report,
                                                                               dry_run=dry_run))


def destroy_dataset(pid, dataset_api, csv_report, dry_run: bool):
    print("Destroying dataset {}".format(pid))
    dataset_api.destroy(dry_run=dry_run)
    action = "Destroyed"
    csv_report.write({'DOI': pid, 'Modified': datetime.now(), 'Change': action})


def main():
    config = init()

    parser = argparse.ArgumentParser(
        description='Deletes one or more, potentially published, datasets. Requires an API token with superuser '
                    'privileges. Furthermore, the dataverse.safety_latch must be set to OFF.')
    parser.add_argument('pid_or_pid_file', help='The pid or file with pids of the datasets to destroy')
    add_batch_processor_args(parser)
    add_dry_run_arg(parser)
    args = parser.parse_args()

    dataverse_client = DataverseClient(config['dataverse'])
    batch_processor = BatchProcessorWithReport(wait=args.wait, report_file=args.report_file)
    destroy_datasets(args, dataverse_client, batch_processor, dry_run=args.dry_run)


if __name__ == '__main__':
    main()



import argparse

import rich

from datastation.common.config import init
from datastation.common.utils import add_dry_run_arg
from datastation.verifydataset.verify_dataset import VerifyDatasetService


def main():
    config = init()
    verify_dataset_service = VerifyDatasetService(config['verify_dataset'])

    parser = argparse.ArgumentParser(description='Verify metadata of a dataset')
    parser.add_argument('pid', help='The pid of the datasets to verify')
    add_dry_run_arg(parser)

    args = parser.parse_args()
    r = verify_dataset_service.verify_dataset(args.pid, dry_run=args.dry_run)
    rich.print_json(data=r)


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['datastation',
 'datastation.common',
 'datastation.dans_bag',
 'datastation.dataverse',
 'datastation.ingestflow',
 'datastation.managedeposit',
 'datastation.scripts',
 'datastation.verifydataset']

package_data = \
{'': ['*']}

install_requires = \
['dicttoxml>=1.7.4,<2.0.0',
 'lxml>=4.8.0,<5.0.0',
 'psycopg>=3.1.8,<4.0.0',
 'pyYAML>=6.0,<7.0',
 'requests>=2.28.2,<3.0.0',
 'rich>=13.7.0,<14.0.0']

entry_points = \
{'console_scripts': ['dans-bag-validate = datastation.dans_bag_validate:main',
                     'datastation-get-component-versions = '
                     'datastation.datastation_get_component_versions:main',
                     'deposit-clean-data = datastation.deposit_clean_data:main',
                     'deposit-create-report = '
                     'datastation.deposit_create_report:main',
                     'dv-banner = datastation.dv_banner:main',
                     'dv-dataset-delete-draft = '
                     'datastation.dv_dataset_delete_draft:main',
                     'dv-dataset-destroy = datastation.dv_dataset_destroy:main',
                     'dv-dataset-destroy-migration-placeholder = '
                     'datastation.dv_dataset_destroy_migration_placeholder:main',
                     'dv-dataset-find-by-role-assignment = '
                     'datastation.dv_dataset_find_by_role_assignment:main',
                     'dv-dataset-get-attributes = '
                     'datastation.dv_dataset_get_attributes:main',
                     'dv-dataset-get-metadata = '
                     'datastation.dv_dataset_get_metadata:main',
                     'dv-dataset-get-metadata-export = '
                     'datastation.dv_dataset_get_metadata_export:main',
                     'dv-dataset-lock = datastation.dv_dataset_lock:main',
                     'dv-dataset-publish = datastation.dv_dataset_publish:main',
                     'dv-dataset-reindex = datastation.dv_dataset_reindex:main',
                     'dv-dataset-reingest-tabular = '
                     'datastation.dv_dataset_reingest_tabular:main',
                     'dv-dataset-role-assignment = '
                     'datastation.dv_dataset_role_assignment:main',
                     'dv-dataset-update-datacite = '
                     'datastation.dv_dataset_update_datacite:main',
                     'dv-dataset-verify = datastation.dv_dataset_verify:main',
                     'dv-dataverse-root-collect-storage-usage = '
                     'datastation.dv_dataverse_root_collect_storage_usage:main',
                     'dv-notifications-cleanup = '
                     'datastation.dv_notifications_cleanup:main',
                     'dv-user-import = datastation.dv_user_import:main',
                     'ingest-flow = datastation.ingest_flow:main']}

setup_kwargs = {
    'name': 'dans-datastation-tools',
    'version': '0.35.0',
    'description': 'Command line utilities for Data Station application management',
    'long_description': 'None',
    'author': 'DANS-KNAW',
    'author_email': 'None',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



import json as jsonlib
import os
import shutil
import argparse


def add_dry_run_arg(parser):
    parser.add_argument('-d', '--dry-run', action='store_true',
                        help='Do not perform the action, but show what would be done.')


def add_batch_processor_args(parser, report: bool = True):
    parser.add_argument('-w', '--wait-between-items', default=2.0, type=float,
                        help="number of seconds to wait between processing items",
                        dest='wait')
    parser.add_argument('-f', '--fail-fast', dest='fail_fast', action='store_true', help='fail on first error ')
    if report:
        parser.add_argument('-r', '--report-file', default='-', help="the report file, or - for stdout",
                            dest='report_file')


def positive_int_argument_converter(value):
    try:
        ivalue = int(value)

        if ivalue <= 0:
            raise argparse.ArgumentTypeError("value must be a number greater than zero")

        return ivalue
    except ValueError:
        raise argparse.ArgumentTypeError("value must be a number greater than zero")


def print_dry_run_message(method, url, params=None, headers=None, data=None, json=None):
    print("DRY-RUN: only printing command, not sending it...")
    print(f"{method} {url}")
    print(f"headers: {headers}") if headers else None
    print(f"params: {params}") if params else None
    print(f"data: {data}") if data else None
    print(f"json: {jsonlib.dumps(json)}") if json else None
    print()


def expand_path(path):
    return os.path.abspath(os.path.expanduser(path))


def is_sub_path_of(child, parent):
    """
    Returns True if child is a descendant directory of parent, or if child is a file in a descendant directory
    of parent. Returns False otherwise. The existence of child and parent is not checked."""
    absolute_parent = expand_path(parent)
    absolute_dir = expand_path(child)
    return absolute_dir.startswith(absolute_parent)


def has_file_pred(file, pred):
    return pred(file)


def has_dirtree_pred(path, pred):
    """
    Returns True if all files and directories in the directory tree rooted at path satisfy the predicate pred.
    """
    for root, dirs, files in os.walk(path):
        return pred(root) \
            and all(pred(os.path.join(root, d)) for d in dirs) \
            and all(pred(os.path.join(root, f)) for f in files)


def have_subdirs_pred(path, pred):
    """
    Returns True if all subdirectories in the directory tree rooted at path satisfy the predicate pred.
    """
    for root, dirs, files in os.walk(path):
        return all(pred(os.path.join(root, d)) for d in dirs)


def set_permissions(path, dir_mode, file_mode, group):
    """
    Sets the permissions of all files and directories in the directory tree rooted at path to dir_mode and file_mode,
    respectively. The group of all files and directories is set to group.
    """
    for root, dirs, files in os.walk(path):
        shutil.chown(root, group=group)
        for d in [root] + dirs:
            p = os.path.join(root, d)
            os.chmod(p, dir_mode)
            shutil.chown(p, group=group)
        for f in files:
            p = os.path.join(root, f)
            os.chmod(p, file_mode)
            shutil.chown(p, group=group)


def get_size(start_path='.'):
    total_size = 0
    for dirpath, dirnames, filenames in os.walk(start_path):
        for f in filenames:
            # skip if it is symbolic link
            if not os.path.islink(os.path.join(dirpath, f)):
                total_size += os.path.getsize(os.path.join(dirpath, f))
    return total_size


# Return size in human-readable format
def sizeof_fmt(num, suffix='B'):
    for unit in ['', 'Ki', 'Mi', 'Gi', 'Ti', 'Pi', 'Ei', 'Zi']:
        if abs(num) < 1024.0:
            return "%3.1f%s%s" % (num, unit, suffix)
        num /= 1024.0
    return "%.1f%s%s" % (num, 'Yi', suffix)



import os

# https://docs.python.org/3/library/email.html#module-email
# https://docs.python.org/3/library/email.examples.html

class SendMail:
    # def __init__(self):
    #     pass

    @staticmethod
    def send(to_email, subject, message_body, attachment):
        email_template = " -s '{0}' {1}<<EOF {2} \nEOF"
        if attachment is not None:
            os.system("mail" + " -a " + attachment + email_template.format(subject, to_email, message_body))
        else:
            os.system("mail" + email_template.format(subject, to_email, message_body))



import argparse
from datetime import datetime

from datastation.common.batch_processing import BatchProcessor, get_pids, BatchProcessorWithReport
from datastation.common.config import init
from datastation.common.utils import add_batch_processor_args, add_dry_run_arg
from datastation.dataverse.dataverse_client import DataverseClient


def delete_dataset_drafts(args, dataverse_client: DataverseClient, batch_processor: BatchProcessor):
    pids = get_pids(args.pid_or_pid_file)
    batch_processor.process_pids(pids,
                                 lambda pid, csv_report: delete_dataset_draft(pid,
                                                                              dataset_api=dataverse_client.dataset(pid),
                                                                              csv_report=csv_report,
                                                                              dry_run=args.dry_run))


def delete_dataset_draft(pid, dataset_api, csv_report, dry_run: bool):
    action = "None"
    if dataset_api.is_draft():
        print("Deleting dataset draft {}".format(pid))
        dataset_api.delete_draft(dry_run=dry_run)
        action = "Deleted"
    else:
        print("Dataset {} is not a draft".format(pid))
    csv_report.write({'DOI': pid, 'Modified': datetime.now(), 'Change': action})


def main():
    config = init()

    parser = argparse.ArgumentParser(description='Deletes one or more dataset drafts')
    parser.add_argument('pid_or_pid_file', help='The pid or file with pids of the datasets to delete')
    add_batch_processor_args(parser)
    add_dry_run_arg(parser)
    args = parser.parse_args()

    dataverse_client = DataverseClient(config['dataverse'])
    batch_processor = BatchProcessorWithReport(wait=args.wait, report_file=args.report_file)
    delete_dataset_drafts(args, dataverse_client, batch_processor)


if __name__ == '__main__':
    main()



import argparse

import rich
from rich.console import Console
from rich.table import Table

from datastation.common.config import init
from datastation.common.version_info import get_rpm_versions, get_dataverse_version, get_dataverse_build_number, \
    get_payara_version


def main():
    config = init()

    parser = argparse.ArgumentParser(
        description='Gets the version of all Data Station components in this installation.')
    parser.add_argument('--json', dest='json', action='store_true', help='Output as JSON')
    args = parser.parse_args()

    components = get_rpm_versions(config['version_info']['dans_rpm_module_prefix'])
    dataverse_version = get_dataverse_version(config['version_info']['dataverse_application_path'])
    dataverse_build_number = get_dataverse_build_number(config['version_info']['dataverse_application_path'])
    components['dataverse'] = f'{dataverse_version} build {dataverse_build_number}'
    payara_version = get_payara_version(config['version_info']['payara_install_path'])
    components['payara'] = payara_version

    if args.json:
        rich.print(components)
        return
    else:
        table = Table(title="Data Station Component Versions")
        table.add_column("Component")
        table.add_column("Version")
        for component in components:
            table.add_row(component, components[component])
        console = Console()
        console.print(table)


if __name__ == '__main__':
    main()
